@@ -1,10 +1,17 @@
+"""
+$description A privately owned Bulgarian live TV channel.
+$url btvplus.bg
+$type live
+$region Bulgaria
+"""
+
 import logging
 import re
 
 from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
-from streamlink.utils import parse_json
+from streamlink.stream.hls import HLSStream
+from streamlink.utils.parse import parse_json
 
 log = logging.getLogger(__name__)
 
