@@ -1,10 +1,20 @@
+"""
+$description Turkish live TV channels from Ciner Group, including Haberturk TV and Show TV.
+$url showtv.com.tr
+$url haberturk.com
+$url showmax.com.tr
+$url showturk.com.tr
+$url bloomberght.com
+$type live
+"""
+
 import json
 import re
 from urllib.parse import unquote
 
 from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
+from streamlink.stream.hls import HLSStream
 
 
 @pluginmatcher(re.compile(r"""
