@@ -1,3 +1,9 @@
+"""
+$description French free-to-air news channel, providing 24-hour national and global news coverage.
+$url cnews.fr
+$type live, vod
+"""
+
 import re
 
 from streamlink.plugin import Plugin, pluginmatcher
