@@ -1,13 +1,22 @@
+"""
+$description Live local TV channels and video on-demand service. OTT service from FilmOn.
+$url filmon.com
+$type live, vod
+$notes Some VODs are mp4 which may not stream, use -o to download
+"""
+
 import logging
 import re
 import time
 from urllib.parse import urlparse, urlunparse
 
 from streamlink.exceptions import PluginError, StreamError
-from streamlink.plugin import Plugin
+from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream, hls_playlist
-from streamlink.stream.hls import HLSStreamReader, HLSStreamWorker, Sequence
+from streamlink.plugin.api.http_session import TLSSecLevel1Adapter
+from streamlink.stream.hls import HLSStream, HLSStreamReader, HLSStreamWorker, Sequence
+from streamlink.stream.hls_playlist import load as load_hls_playlist
+from streamlink.stream.http import HTTPStream
 
 log = logging.getLogger(__name__)
 
@@ -47,7 +56,7 @@ class FilmOnHLSStreamWorker(HLSStreamWorker):
             raise err
 
         try:
-            playlist = hls_playlist.load(res.text, res.url)
+            playlist = load_hls_playlist(res.text, res.url)
         except ValueError as err:
             raise StreamError(err)
 
@@ -114,7 +123,7 @@ class FilmOnHLS(HLSStream):
         url = self.url
         expires = self.watch_timeout - time.time()
         if expires < 0:
-            raise TypeError("Stream has expired and cannot be converted to a URL")
+            raise TypeError("Stream has expired and cannot be translated to a URL")
         return url
 
 
@@ -122,24 +131,32 @@ class FilmOnAPI:
     def __init__(self, session):
         self.session = session
 
-    channel_url = "http://www.filmon.com/api-v2/channel/{0}?protocol=hls"
-    vod_url = "http://www.filmon.com/vod/info/{0}"
+    channel_url = "https://www.filmon.com/ajax/getChannelInfo"
+    vod_url = "https://vms-admin.filmon.com/api/video/movie?id={0}"
 
     stream_schema = {
-        "quality": validate.text,
+        "quality": str,
         "url": validate.url(),
         "watch-timeout": int
     }
-    api_schema = validate.Schema(
+    channel_schema = validate.Schema(
         {
-            "data": {
+            "streams": validate.any(
+                {str: stream_schema},
+                [stream_schema]
+            )
+        }
+    )
+    vod_schema = validate.Schema(
+        {
+            "response": {
                 "streams": validate.any(
-                    {validate.text: stream_schema},
+                    {str: stream_schema},
                     [stream_schema]
                 )
             }
         },
-        validate.get("data")
+        validate.get("response")
     )
 
     def channel(self, channel):
@@ -150,11 +167,13 @@ class FilmOnAPI:
 
             # retry for 50X errors
             try:
-                res = self.session.http.get(self.channel_url.format(channel))
+                res = self.session.http.post(self.channel_url,
+                                             data={"channel_id": channel, "quality": "low"},
+                                             headers={"X-Requested-With": "XMLHttpRequest"})
                 if res:
                     # retry for invalid response data
                     try:
-                        return self.session.http.json(res, schema=self.api_schema)
+                        return self.session.http.json(res, schema=self.channel_schema)
                     except PluginError:
                         log.debug("invalid or non-JSON data received")
                         continue
@@ -165,15 +184,16 @@ class FilmOnAPI:
 
     def vod(self, vod_id):
         res = self.session.http.get(self.vod_url.format(vod_id))
-        return self.session.http.json(res, schema=self.api_schema)
+        return self.session.http.json(res, schema=self.vod_schema)
 
 
-class Filmon(Plugin):
-    url_re = re.compile(r"""(?x)https?://(?:www\.)?filmon\.(?:tv|com)/(?:
+@pluginmatcher(re.compile(r"""
+    https?://(?:www\.)?filmon\.(?:tv|com)/
+    (?:
         (?:
             index/popout\?
             |
-            (?:tv/)channel/(?:export\?)?
+            (?:tv/)?channel/(?:export\?)?
             |
             tv/(?!channel/)
             |
@@ -181,10 +201,11 @@ class Filmon(Plugin):
             |
             (?P<is_group>group/)
         )(?:channel_id=)?(?P<channel>[-_\w]+)
-    |
-        vod/view/(?P<vod_id>\d+)-
-    )""")
-
+        |
+        vod/view/(?P<vod_id>[^/?&]+)
+    )
+""", re.VERBOSE))
+class Filmon(Plugin):
     _channel_id_re = re.compile(r"""channel_id\s*=\s*(?P<quote>['"]?)(?P<value>\d+)(?P=quote)""")
     _channel_id_schema = validate.Schema(
         validate.transform(_channel_id_re.search),
@@ -205,10 +226,6 @@ class Filmon(Plugin):
             self.url = urlunparse(parsed._replace(path=parsed.path.replace("/channel/", "/tv/")))
         self.api = FilmOnAPI(self.session)
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls.url_re.match(url) is not None
-
     @classmethod
     def stream_weight(cls, key):
         weight = cls.quality_weights.get(key)
@@ -218,20 +235,32 @@ class Filmon(Plugin):
         return Plugin.stream_weight(key)
 
     def _get_streams(self):
-        url_m = self.url_re.match(self.url)
+        channel = self.match.group("channel")
+        vod_id = self.match.group("vod_id")
+        is_group = self.match.group("is_group")
 
-        channel = url_m and url_m.group("channel")
-        vod_id = url_m and url_m.group("vod_id")
-        is_group = url_m and url_m.group("is_group")
+        adapter = TLSSecLevel1Adapter()
+        self.session.http.mount("https://filmon.com", adapter)
+        self.session.http.mount("https://www.filmon.com", adapter)
+        self.session.http.mount("https://vms-admin.filmon.com/", adapter)
+
+        # get cookies
+        self.session.http.get(self.url)
 
         if vod_id:
             data = self.api.vod(vod_id)
             for _, stream in data["streams"].items():
-                streams = HLSStream.parse_variant_playlist(self.session, stream["url"])
-                if not streams:
-                    yield stream["quality"], HLSStream(self.session, stream["url"])
+                if stream["url"].endswith(".m3u8"):
+                    streams = HLSStream.parse_variant_playlist(self.session, stream["url"])
+                    if not streams:
+                        yield stream["quality"], HLSStream(self.session, stream["url"])
+                    else:
+                        yield from streams.items()
+                elif stream["url"].endswith(".mp4"):
+                    yield stream["quality"], HTTPStream(self.session, stream["url"])
                 else:
-                    yield from streams.items()
+                    log.error("Unsupported stream type")
+                    return
         else:
             if channel and not channel.isdigit():
                 _id = self.cache.get(channel)
