@@ -1,10 +1,17 @@
+"""
+$description Israeli live TV channel and video on-demand service owned by Network 13.
+$url 13tv.co.il
+$type live, vod
+$region Israel
+"""
+
 import logging
 import re
 from urllib.parse import urljoin, urlunparse
 
 from streamlink.plugin import Plugin, PluginError, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
+from streamlink.stream.hls import HLSStream
 
 log = logging.getLogger(__name__)
 
