@@ -1,8 +1,16 @@
+"""
+$description American video on-demand service owned by NBC Network.
+$url nbc.com
+$type vod
+$region United States of America
+$account Authentication is unsupported
+"""
+
 import re
 
 from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugins.theplatform import ThePlatform
-from streamlink.utils import update_scheme
+from streamlink.utils.url import update_scheme
 
 
 @pluginmatcher(re.compile(
@@ -17,7 +25,7 @@ class NBC(Plugin):
         platform_url = m and m.group("url")
 
         if platform_url:
-            url = update_scheme(self.url, platform_url)
+            url = update_scheme("https://", platform_url)
             # hand off to ThePlatform plugin
             p = ThePlatform(url)
             p.bind(self.session, "plugin.nbc")
