@@ -1,3 +1,9 @@
+"""
+$description Chinese live streaming platform for live video game broadcasts and live sports game related streams.
+$url live.qq.com
+$type live
+"""
+
 import logging
 import re
 
