@@ -1,8 +1,14 @@
+"""
+$description Polish live TV channel owned by Toya.
+$url tvtoya.pl
+$type live
+"""
+
 import logging
 import re
 
 from streamlink.plugin import Plugin, pluginmatcher
-from streamlink.stream import HLSStream
+from streamlink.stream.hls import HLSStream
 
 log = logging.getLogger(__name__)
 
